@@ -5,7 +5,7 @@ __license__ = "Apache-2.0"
 def py_client(**kwargs):
     """A simple Python client for connecting to the gateway.
 
-    For acceptable ``kwargs``, please refer to :cmd:`jina client --help`
+    For acceptable ``kwargs``, please refer to :command:`jina client --help`
 
     Example, assuming a Flow is "standby" on 192.168.1.100, with port_grpc at 55555.
 
@@ -15,13 +15,13 @@ def py_client(**kwargs):
         from jina.clients import py_client
 
         # to test connectivity
-        py_client(port_grpc='192.168.1.100', host=55555).dry_run()
+        py_client(host='192.168.1.100', port_grpc=55555).dry_run()
 
         # to search
-        py_client(port_grpc='192.168.1.100', host=55555).search(input_fn, output_fn)
+        py_client(host='192.168.1.100', port_grpc=55555).search(input_fn, output_fn)
 
         # to index
-        py_client(port_grpc='192.168.1.100', host=55555).index(input_fn, output_fn)
+        py_client(host='192.168.1.100', port_grpc=55555).index(input_fn, output_fn)
     """
     from ..main.parser import set_client_cli_parser
     from ..helper import get_parsed_args
