@@ -6,6 +6,7 @@ from typing import Iterator, Callable, Union
 from . import request
 from .grpc import GrpcClient
 from .helper import ProgressBar
+from ...enums import ClientMode
 from ...excepts import BadClient
 from ...logging import default_logger
 from ...logging.profile import TimeContext
@@ -29,13 +30,13 @@ class PyClient(GrpcClient):
         from jina.clients import py_client
 
         # to test connectivity
-        py_client(port_grpc='192.168.1.100', host=55555).dry_run()
+        py_client(host='192.168.1.100', port_grpc=55555).dry_run()
 
         # to search
-        py_client(port_grpc='192.168.1.100', host=55555).search(input_fn, output_fn)
+        py_client(host='192.168.1.100', port_grpc=55555).search(input_fn, output_fn)
 
         # to index
-        py_client(port_grpc='192.168.1.100', host=55555).index(input_fn, output_fn)
+        py_client(host='192.168.1.100', port_grpc=55555).index(input_fn, output_fn)
 
     """
 
@@ -43,8 +44,7 @@ class PyClient(GrpcClient):
         """
 
         :param args: args provided by the CLI
-        :param delay: if ``True`` then the client starts sending request after initializing, otherwise one needs to set
-            the :attr:`input_fn` before using :func:`start` or :func:`call`
+
         """
         super().__init__(args)
         self._mode = self.args.mode
@@ -55,32 +55,35 @@ class PyClient(GrpcClient):
         return self._mode
 
     @mode.setter
-    def mode(self, value):
-        avail = {'train', 'index', 'search'}
-        if value in avail:
+    def mode(self, value: ClientMode):
+        if isinstance(value, ClientMode):
             self._mode = value
             self.args.mode = value
         else:
-            raise ValueError(f'{value} must be one of {avail}')
+            raise ValueError(f'{value} must be one of {ClientMode}')
 
     @staticmethod
-    def check_input(input_fn: Union[Iterator['jina_pb2.Document'], Iterator[bytes], Callable] = None,
-                    in_proto: bool = False):
+    def check_input(input_fn: Union[Iterator['jina_pb2.Document'], Iterator[bytes], Callable] = None):
         """Validate the input_fn and print the first request if success
 
         :param input_fn: the input function
-        :param in_proto: if the input data is already in protobuf Document format, or in raw bytes
         """
-        kwargs = {'data': input_fn, 'in_proto': in_proto}
+        if hasattr(input_fn, '__call__'):
+            input_fn = input_fn()
+
+        kwargs = {'data': input_fn}
 
         try:
             r = next(getattr(request, 'index')(**kwargs))
-            default_logger.success(f'input_fn is valid and the first request is as follows:\n{r}')
+            if r is not None:
+                default_logger.success(f'input_fn is valid and the first request is as follows:\n{r}')
+            else:
+                raise TypeError
         except:
             default_logger.error(f'input_fn is not valid!')
             raise
 
-    def call_unary(self, data: Union['jina_pb2.Document', bytes], mode: str) -> None:
+    def call_unary(self, data: Union['jina_pb2.Document', bytes], mode: ClientMode) -> None:
         """ Calling the server with one request only, and return the result
 
         This function should not be used in production due to its low-efficiency. For example,
@@ -94,7 +97,7 @@ class PyClient(GrpcClient):
         kwargs = vars(self.args)
         kwargs['data'] = [data]
 
-        req_iter = getattr(request, self.mode)(**kwargs)
+        req_iter = getattr(request, str(self.mode).lower())(**kwargs)
         return self._stub.CallUnary(next(req_iter))
 
     def call(self, callback: Callable[['jina_pb2.Message'], None] = None, **kwargs) -> None:
@@ -110,15 +113,24 @@ class PyClient(GrpcClient):
             if k in kwargs:
                 _kwargs[k] = kwargs[k]
 
-        tname = self.mode
+        tname = str(self.mode).lower()
         if 'mode' in kwargs:
-            tname = kwargs['mode']
+            tname = str(kwargs['mode']).lower()
+
+        if 'mime_type' not in kwargs:
+            self.logger.warning('starting from v0.2.0, '
+                                'the best practice of sending binary data is with "mime_type". '
+                                'when not given then MIME sniff (based on libmagic) will be used')
 
         req_iter = getattr(request, tname)(**_kwargs)
         # next(req_iter)
 
         with ProgressBar(task_name=tname) as p_bar, TimeContext(tname):
             for resp in self._stub.Call(req_iter):
+                if resp.status.code >= jina_pb2.Status.ERROR:
+                    self.logger.error(f'callback() may not work properly '
+                                      f'due to the bad response: {resp.status.description}')
+                    self.logger.error(resp.status.details)
                 if callback:
                     try:
                         if self.args.callback_on_body:
@@ -131,9 +143,9 @@ class PyClient(GrpcClient):
     @property
     def input_fn(self) -> Union[Iterator['jina_pb2.Document'], Iterator[bytes], Callable]:
         """ An iterator of bytes, each element represents a document's raw content,
-        i.e. ``input_fn`` defined int the protobuf
+        i.e. ``input_fn`` defined in the protobuf
         """
-        if self._input_fn:
+        if self._input_fn is not None:
             return self._input_fn
         else:
             raise BadClient('input_fn is empty or not set')
@@ -148,7 +160,7 @@ class PyClient(GrpcClient):
             self._input_fn = bytes_gen
 
     def dry_run(self) -> bool:
-        """Send a DRYRUN request to the server, passing through all pods on the server
+        """Send a DRYRUN request to the server, passing through all pods on the server,
         useful for testing connectivity and debugging
 
         :return: if dry run is successful or not
@@ -167,18 +179,18 @@ class PyClient(GrpcClient):
 
     def train(self, input_fn: Union[Iterator['jina_pb2.Document'], Iterator[bytes], Callable] = None,
               output_fn: Callable[['jina_pb2.Message'], None] = None, **kwargs):
-        self.mode = 'train'
+        self.mode = ClientMode.TRAIN
         self.input_fn = input_fn
         self.start(output_fn, **kwargs)
 
     def search(self, input_fn: Union[Iterator['jina_pb2.Document'], Iterator[bytes], Callable] = None,
                output_fn: Callable[['jina_pb2.Message'], None] = None, **kwargs):
-        self.mode = 'search'
+        self.mode = ClientMode.SEARCH
         self.input_fn = input_fn
         self.start(output_fn, **kwargs)
 
     def index(self, input_fn: Union[Iterator['jina_pb2.Document'], Iterator[bytes], Callable] = None,
               output_fn: Callable[['jina_pb2.Message'], None] = None, **kwargs):
-        self.mode = 'index'
+        self.mode = ClientMode.INDEX
         self.input_fn = input_fn
         self.start(output_fn, **kwargs)
