@@ -1,8 +1,11 @@
 __copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
+import mimetypes
 import os
-from typing import Dict, Any, Iterable, Tuple
+import urllib.parse
+import urllib.request
+from typing import Dict, Any, Iterable, Tuple, Union, List, Callable
 
 import numpy as np
 
@@ -17,7 +20,7 @@ def pb2array(blob: 'jina_pb2.NdArray') -> 'np.ndarray':
 
     :param blob: a blob described in protobuf
     """
-    x = np.frombuffer(blob.raw_bytes, dtype=blob.dtype)
+    x = np.frombuffer(blob.buffer, dtype=blob.dtype)
 
     if blob.quantization == jina_pb2.NdArray.FP16:
         x = x.astype(blob.original_dtype)
@@ -61,19 +64,23 @@ def array2pb(x: 'np.ndarray', quantize: str = None) -> 'jina_pb2.NdArray':
     else:
         blob.quantization = jina_pb2.NdArray.NONE
 
-    blob.raw_bytes = x.tobytes()
+    blob.buffer = x.tobytes()
     blob.shape.extend(list(x.shape))
     blob.dtype = x.dtype.name
     return blob
 
 
-def extract_chunks(docs: Iterable['jina_pb2.Document'], embedding: bool) -> Tuple:
+def extract_chunks(docs: Iterable['jina_pb2.Document'],
+                   chunks_iter_fn: Callable,
+                   filter_by: Union[Tuple[str], List[str]],
+                   embedding: bool) -> Tuple:
     """Iterate over a list of protobuf documents and extract chunk-level information from them
 
     :param docs: an iterable of protobuf documents
     :param embedding: an indicator of extracting embedding or not.
                     If ``True`` then all chunk-level embedding are extracted.
-                    If ``False`` then ``text``, ``raw_bytes``, ``blob`` info of each chunks are extracted
+                    If ``False`` then ``text``, ``buffer``, ``blob`` info of each chunks are extracted
+    :param filter_by: a list of service names to wait
     :return: A tuple of four pieces:
 
             - a numpy ndarray of extracted info
@@ -81,30 +88,34 @@ def extract_chunks(docs: Iterable['jina_pb2.Document'], embedding: bool) -> Tupl
             - the doc_id list where the doc has no chunk, useful for debugging
             - the chunk_id list where the chunk has no contents, useful for debugging
     """
-    contents = []
+    _contents = []
     chunk_pts = []
     no_chunk_docs = []
     bad_chunk_ids = []
 
     if embedding:
-        _extract_fn = lambda c: c.embedding.raw_bytes and pb2array(c.embedding)
+        _extract_fn = lambda c: c.embedding.buffer and pb2array(c.embedding)
     else:
-        _extract_fn = lambda c: c.text or c.raw_bytes or (c.blob and pb2array(c.blob))
+        _extract_fn = lambda c: c.text or c.buffer or (c.blob and pb2array(c.blob))
 
     for d in docs:
-        if not d.chunks:
-            no_chunk_docs.append(d.doc_id)
-            continue
-
-        for c in d.chunks:
+        has_chunk = False
+        for c in chunks_iter_fn(d):
             _c = _extract_fn(c)
+            if filter_by and (c.field_name not in filter_by):
+                continue
+
             if _c is not None:
-                contents.append(_c)
+                _contents.append(_c)
                 chunk_pts.append(c)
+                has_chunk = True
             else:
                 bad_chunk_ids.append((d.doc_id, c.chunk_id))
+        if not has_chunk:
+            no_chunk_docs.append(d.doc_id)
 
-    return np.stack(contents), chunk_pts, no_chunk_docs, bad_chunk_ids
+    contents = np.stack(_contents) if len(_contents) > 0 else None
+    return contents, chunk_pts, no_chunk_docs, bad_chunk_ids
 
 
 def routes2str(msg: 'jina_pb2.Message', flag_current: bool = False) -> str:
@@ -140,3 +151,14 @@ def pb_obj2dict(obj, keys: Iterable[str]) -> Dict[str, Any]:
     :param keys: an iterable of keys for extraction
     """
     return {k: getattr(obj, k) for k in keys if hasattr(obj, k)}
+
+
+def guess_mime(uri):
+    # guess when uri points to a local file
+    m_type = mimetypes.guess_type(uri)[0]
+    # guess when uri points to a remote file
+    if not m_type and urllib.parse.urlparse(uri).scheme in {'http', 'https', 'data'}:
+        page = urllib.request.Request(uri, headers={'User-Agent': 'Mozilla/5.0'})
+        tmp = urllib.request.urlopen(page)
+        m_type = tmp.info().get_content_type()
+    return m_type
