@@ -1,6 +1,7 @@
 __copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
+import io
 from typing import Dict, List
 
 import numpy as np
@@ -22,20 +23,25 @@ class ImageReader(BaseSegmenter):
         super().__init__(*args, **kwargs)
         self.channel_axis = channel_axis
 
-    def craft(self, raw_bytes: bytes, doc_id: int, *args, **kwargs) -> List[Dict]:
+    def craft(self, buffer: bytes, uri: str, doc_id: int, *args, **kwargs) -> List[Dict]:
         """
-        Read the image from the given file path that specified in `raw_bytes` and save the `ndarray` of the image in
+        Read the image from the given file path that specified in `buffer` and save the `ndarray` of the image in
             the `blob` of the chunk.
 
-        :param raw_bytes: the image file path in raw bytes
+        :param buffer: the image in raw bytes
+        :param uri: the image file path
         :param doc_id: the id of the Document
 
         """
         from PIL import Image
-        raw_img = Image.open(raw_bytes.decode())
-        if raw_img.mode != 'RGB':
-            raw_img = raw_img.convert('RGB')
+        if buffer:
+            raw_img = Image.open(io.BytesIO(buffer))
+        elif uri:
+            raw_img = Image.open(uri)
+        else:
+            raise ValueError('no value found in "buffer" and "uri"')
+        raw_img = raw_img.convert('RGB')
         img = np.array(raw_img).astype('float32')
         if self.channel_axis != -1:
             img = np.moveaxis(img, -1, self.channel_axis)
-        return [dict(doc_id=doc_id, offset=0, weight=1., blob=img), ]
+        return [dict(offset=0, weight=1., blob=img)]
