@@ -40,8 +40,14 @@ class BaseIndexer(BaseExecutor):
     """
 
     def __init__(self,
-                 index_filename: str,
+                 index_filename: str = None,
                  *args, **kwargs):
+        """
+
+        :param index_filename: the name of the file for storing the index, when not given metas.name is used.
+        :param args:
+        :param kwargs:
+        """
         super().__init__(*args, **kwargs)
         self.index_filename = index_filename  #: the file name of the stored index, no path is required
         self._size = 0
@@ -52,10 +58,10 @@ class BaseIndexer(BaseExecutor):
         :param keys: ``chunk_id`` in 1D-ndarray, shape B x 1
         :param vectors: vector representations in B x D
         """
-        pass
 
     def post_init(self):
         """query handler and write handler can not be serialized, thus they must be put into :func:`post_init`. """
+        self.index_filename = self.index_filename or self.name
         self._query_handler = None
         self._write_handler = None
 
@@ -67,7 +73,7 @@ class BaseIndexer(BaseExecutor):
         :return: a tuple of two ndarray.
             The first is ids in shape B x K (`dtype=int`), the second is scores in shape B x K (`dtype=float`)
         """
-        pass
+        raise NotImplementedError
 
     @property
     def index_abspath(self) -> str:
@@ -81,6 +87,7 @@ class BaseIndexer(BaseExecutor):
         """A readable and indexable object, could be dict, map, list, numpy array etc. """
         if self._query_handler is None and os.path.exists(self.index_abspath):
             self._query_handler = self.get_query_handler()
+            self.logger.info(f'indexer size: {self.size}')
 
         if self._query_handler is None:
             self.logger.warning(f'you can not query from {self} as its "query_handler" is not set. '
@@ -127,6 +134,7 @@ class BaseIndexer(BaseExecutor):
 
     def close(self):
         """Close all file-handlers and release all resources. """
+        self.logger.info(f'indexer size: {self.size}')
         self.flush()
         call_obj_fn(self._write_handler, 'close')
         call_obj_fn(self._query_handler, 'close')
@@ -161,11 +169,11 @@ class ChunkIndexer(CompoundExecutor):
 
         -  In the index time
             - 1. stores the vector via :class:`BaseVectorIndexer`
-            - 2. remove all vector information (embedding, raw_bytes, blob, text)
+            - 2. remove all vector information (embedding, buffer, blob, text)
             - 3. store the remained meta information via :class:`BaseKVIndexer`
         - In the query time
             - 1. Find the knn using the vector via :class:`BaseVectorIndexer`
-            - 2. remove all vector information (embedding, raw_bytes, blob, text)
+            - 2. remove all vector information (embedding, buffer, blob, text)
             - 3. Fill in the meta information of the chunk via :class:`BaseKVIndexer`
 
     One can use the :class:`ChunkIndexer` via
@@ -206,7 +214,7 @@ class ChunkIndexer(CompoundExecutor):
                 level: chunk
                 pruned:
                   - embedding
-                  - raw_bytes
+                  - buffer
                   - blob
                   - text
             - !KVSearchDriver
@@ -222,7 +230,7 @@ class ChunkIndexer(CompoundExecutor):
                 level: chunk
                 pruned:
                   - embedding
-                  - raw_bytes
+                  - buffer
                   - blob
                   - text
             - !KVIndexDriver
@@ -232,5 +240,3 @@ class ChunkIndexer(CompoundExecutor):
           ControlRequest:
             - !ControlReqDriver {}
     """
-
-    pass
