@@ -5,6 +5,7 @@ import json
 from typing import Union
 
 from google.protobuf.json_format import Parse
+
 from jina.executors.indexers.keyvalue.proto import BasePbIndexer
 from jina.executors.indexers.keyvalue.proto import jina_pb2
 
@@ -14,28 +15,19 @@ class LeveldbIndexer(BasePbIndexer):
     :class:`LeveldbIndexer` use `LevelDB` to save and query protobuf chunk/document.
     """
 
-    def post_init(self):
-        super().post_init()
-        self._db_handler = None
-
-    @property
-    def db_handler(self):
-        if self._db_handler is None:
-            import plyvel
-            self._db_handler = plyvel.DB(self.index_abspath, create_if_missing=True)
-        return self._db_handler
-
     def get_add_handler(self):
         """Get the database handler
 
         """
-        return self.db_handler
+        import plyvel
+        return plyvel.DB(self.index_abspath, create_if_missing=True)
 
     def get_create_handler(self):
         """Get the database handler
 
         """
-        return self.db_handler
+        import plyvel
+        return plyvel.DB(self.index_abspath, create_if_missing=True)
 
     def add(self, objs):
         """Add a JSON-friendly object to the indexer
@@ -52,7 +44,8 @@ class LeveldbIndexer(BasePbIndexer):
         """Get the database handler
 
         """
-        return self.db_handler
+        import plyvel
+        return plyvel.DB(self.index_abspath, create_if_missing=True)
 
     def query(self, key: str, *args, **kwargs) -> Union['jina_pb2.Chunk', 'jina_pb2.Document']:
         """Find the protobuf chunk/doc using id
@@ -63,17 +56,9 @@ class LeveldbIndexer(BasePbIndexer):
         v = self.query_handler.get(key.encode('utf8'))
         value = None
         if v is not None:
-            _parser = jina_pb2.Chunk if key[0] == 'c' else jina_pb2.Document
-            value = Parse(json.loads(v.decode('utf8')), _parser())
+            value = Parse(json.loads(v.decode('utf8')), self._parser())
         return value
 
-    def close(self):
-        """Close the database handler
-
-        """
-        super().close()
-        self.write_handler.close()
-
 
 class ChunkLeveldbIndexer(LeveldbIndexer):
     """"""
