@@ -25,10 +25,11 @@ from ..peapods.pod import SocketType, FlowPod, GatewayFlowPod
 if False:
     from ..proto import jina_pb2
     import argparse
+    import numpy as np
 
 
 def build_required(required_level: 'FlowBuildLevel'):
-    """Annotate a function so that it requires certaidn build level to run.
+    """Annotate a function so that it requires certain build level to run.
 
     :param required_level: required build level to run this function.
 
@@ -54,7 +55,7 @@ def build_required(required_level: 'FlowBuildLevel'):
                         'build_level check failed for %r, required level: %s, actual level: %s' % (
                             func, required_level, self._build_level))
             else:
-                raise AttributeError('%r has no attribute "_build_level"' % self)
+                raise AttributeError(f'{self!r} has no attribute "_build_level"')
 
         return arg_wrapper
 
@@ -70,7 +71,7 @@ class Flow:
 
         More explain on ``optimize_level``:
 
-        As an example, the following flow will generates a 6 Peas,
+        As an example, the following flow will generate 6 Peas,
 
         .. highlight:: python
         .. code-block:: python
@@ -80,6 +81,7 @@ class Flow:
         The optimized version, i.e. :code:`Flow(optimize_level=FlowOptimizeLevel.FULL)`
         will generate 4 Peas, but it will force the :class:`GatewayPea` to take BIND role,
         as the head and tail routers are removed.
+        
         """
         self.logger = get_logger(self.__class__.__name__)
         self._pod_nodes = OrderedDict()  # type: Dict[str, 'FlowPod']
@@ -229,7 +231,7 @@ class Flow:
                 if s == pod_name:
                     raise FlowTopologyError('the income/output of a pod can not be itself')
         else:
-            raise ValueError('endpoint=%s is not parsable' % endpoint)
+            raise ValueError(f'endpoint={endpoint} is not parsable')
         return set(endpoint)
 
     def set_last_pod(self, name: str, copy_flow: bool = True) -> 'Flow':
@@ -243,7 +245,7 @@ class Flow:
         op_flow = copy.deepcopy(self) if copy_flow else self
 
         if name not in op_flow._pod_nodes:
-            raise FlowMissingPodError('%s can not be found in this Flow' % name)
+            raise FlowMissingPodError(f'{name} can not be found in this Flow')
 
         if op_flow._last_changed_pod and name == op_flow._last_changed_pod[-1]:
             pass
@@ -267,7 +269,7 @@ class Flow:
 
     def join(self, needs: Union[Tuple[str], List[str]], *args, **kwargs) -> 'Flow':
         """
-        Add a blocker to the flow, wait until all peas defined in `needs` completed.
+        Add a blocker to the flow, wait until all peas defined in **needs** completed.
 
         :param needs: list of service names to wait
         :return: the modified flow
@@ -300,7 +302,7 @@ class Flow:
         pod_name = kwargs.get('name', None)
 
         if pod_name in op_flow._pod_nodes:
-            raise FlowTopologyError('name: %s is used in this Flow already!' % pod_name)
+            raise FlowTopologyError(f'name: {pod_name} is used in this Flow already!')
 
         if not pod_name:
             pod_name = '%s%d' % ('pod', op_flow._pod_name_counter)
@@ -308,20 +310,19 @@ class Flow:
 
         if not pod_name.isidentifier():
             # hyphen - can not be used in the name
-            raise ValueError('name: %s is invalid, please follow the python variable name conventions' % pod_name)
+            raise ValueError(f'name: {pod_name} is invalid, please follow the python variable name conventions')
 
         needs = op_flow._parse_endpoints(op_flow, pod_name, needs, connect_to_last_pod=True)
 
         kwargs.update(op_flow._common_kwargs)
         kwargs['name'] = pod_name
-        kwargs['num_part'] = len(needs)
         op_flow._pod_nodes[pod_name] = FlowPod(kwargs=kwargs, needs=needs)
 
         op_flow.set_last_pod(pod_name, False)
 
         return op_flow
 
-    def build(self, inplace: bool = True) -> 'Flow':
+    def build(self, copy_flow: bool = False) -> 'Flow':
         """
         Build the current flow and make it ready to use
 
@@ -330,7 +331,7 @@ class Flow:
             No need to manually call it since 0.0.8. When using flow with the
             context manager, or using :meth:`start`, :meth:`build` will be invoked.
 
-        :param inplace: if set to ``False`` then return the copy of the current flow.
+        :param copy_flow: when set to true, then always copy the current flow and do the modification on top of it then return, otherwise, do in-line modification
         :return: the current flow (by default)
 
         .. note::
@@ -343,12 +344,12 @@ class Flow:
                 with f:
                     f.index()
 
-                with f.build(copy_flow=False) as fl:
+                with f.build(copy_flow=True) as fl:
                     fl.search()
 
         """
 
-        op_flow = self if inplace else copy.deepcopy(self)
+        op_flow = copy.deepcopy(self) if copy_flow else self
 
         _pod_edges = set()
 
@@ -359,8 +360,8 @@ class Flow:
         for k, p in op_flow._pod_nodes.items():
             for s in p.needs:
                 if s not in op_flow._pod_nodes:
-                    raise FlowMissingPodError('%s is not in this flow, misspelled name?' % s)
-                _pod_edges.add('%s-%s' % (s, k))
+                    raise FlowMissingPodError(f'{s} is not in this flow, misspelled name?')
+                _pod_edges.add(f'{s}-{k}')
 
         for k in _pod_edges:
             s_name, e_name = k.split('-')
@@ -395,7 +396,7 @@ class Flow:
                     else:
                         FlowPod.connect(s_pod, e_pod, first_socket_type=SocketType.PUSH_CONNECT)
                 elif e_name == 'gateway':
-                    if self.args.optimize_level > FlowOptimizeLevel.IGNORE_GATEWAY and s_pod.is_tail_router and s_pod.tail_args.num_part == 1:
+                    if self.args.optimize_level > FlowOptimizeLevel.IGNORE_GATEWAY and s_pod.is_tail_router and s_pod.tail_args.num_part <= 1:
                         # connect gateway directly to peas only if this is unblock router
                         # as gateway can not block & reduce message
                         s_pod.connect_to_head_of(e_pod)
@@ -405,7 +406,7 @@ class Flow:
                     e_pod.head_args.socket_in = s_pod.tail_args.socket_out.paired
                     if self.args.optimize_level > FlowOptimizeLevel.NONE and e_pod.is_head_router and not s_pod.is_tail_router:
                         e_pod.connect_to_tail_of(s_pod)
-                    elif self.args.optimize_level > FlowOptimizeLevel.NONE and s_pod.is_tail_router and s_pod.tail_args.num_part == 1:
+                    elif self.args.optimize_level > FlowOptimizeLevel.NONE and s_pod.is_tail_router and s_pod.tail_args.num_part <= 1:
                         s_pod.connect_to_head_of(e_pod)
                     else:
                         FlowPod.connect(s_pod, e_pod, first_socket_type=SocketType.PUSH_CONNECT)
@@ -456,7 +457,7 @@ class Flow:
         """
 
         if self._build_level.value < FlowBuildLevel.GRAPH.value:
-            self.build(inplace=True)
+            self.build(copy_flow=False)
 
         if self.args.logserver:
             self.logger.info('start logserver...')
@@ -470,7 +471,7 @@ class Flow:
             self.num_pods,
             self.num_peas))
 
-        self.logger.success('flow is now ready for use, current build_level is %s' % self._build_level)
+        self.logger.success(f'flow is now ready for use, current build_level is {self._build_level}')
 
         return self
 
@@ -493,7 +494,7 @@ class Flow:
         self._build_level = FlowBuildLevel.EMPTY
         # time.sleep(1)  # sleep for a while until all resources are safely closed
         self.logger.success(
-            'flow is closed and all resources should be released already, current build level is %s' % self._build_level)
+            f'flow is closed and all resources should be released already, current build level is {self._build_level}')
 
     def __eq__(self, other: 'Flow'):
         """
@@ -519,9 +520,13 @@ class Flow:
     def _get_client(self, **kwargs):
         kwargs.update(self._common_kwargs)
         from ..clients import py_client
-        return py_client(port_grpc=self.port_grpc, host=self.host, **kwargs)
+        if 'port_grpc' not in kwargs:
+            kwargs['port_grpc'] = self.port_grpc
+        if 'host' not in kwargs:
+            kwargs['host'] = self.host
+        return py_client(**kwargs)
 
-    @deprecated_alias(raw_bytes='input_fn', callback='output_fn')
+    @deprecated_alias(buffer='input_fn', callback='output_fn')
     def train(self, input_fn: Union[Iterator['jina_pb2.Document'], Iterator[bytes], Callable] = None,
               output_fn: Callable[['jina_pb2.Message'], None] = None,
               **kwargs):
@@ -534,10 +539,8 @@ class Flow:
         .. highlight:: python
         .. code-block:: python
 
-            with f.build(runtime='thread') as flow:
-                flow.train(txt_file='aa.txt')
-                flow.train(image_zip_file='aa.zip', batch_size=64)
-                flow.train(video_zip_file='aa.zip')
+            with f:
+                f.train(input_fn)
                 ...
 
 
@@ -557,13 +560,117 @@ class Flow:
             with f.build(runtime='thread') as flow:
                 flow.train(bytes_gen=my_reader())
 
-        :param input_fn: An iterator of bytes. If not given, then you have to specify it in `kwargs`.
+        :param input_fn: An iterator of bytes. If not given, then you have to specify it in **kwargs**.
         :param output_fn: the callback function to invoke after training
         :param kwargs: accepts all keyword arguments of `jina client` CLI
         """
         self._get_client(**kwargs).train(input_fn, output_fn)
 
-    @deprecated_alias(raw_bytes='input_fn', callback='output_fn')
+    def index_numpy(self, array: 'np.ndarray', axis: int = 0, size: int = None, shuffle: bool = False,
+                    output_fn: Callable[['jina_pb2.Message'], None] = None,
+                    **kwargs):
+        """Using numpy ndarray as the index source for the current flow
+
+        :param array: the numpy ndarray data source
+        :param axis: iterate over that axis
+        :param size: the maximum number of the sub arrays
+        :param shuffle: shuffle the the numpy data source beforehand
+        :param output_fn: the callback function to invoke after indexing
+        :param kwargs: accepts all keyword arguments of `jina client` CLI
+        """
+        from ..clients.python.io import input_numpy
+        self._get_client(**kwargs).index(input_numpy(array, axis, size, shuffle), output_fn)
+
+    def search_numpy(self, array: 'np.ndarray', axis: int = 0, size: int = None, shuffle: bool = False,
+                     output_fn: Callable[['jina_pb2.Message'], None] = None,
+                     **kwargs):
+        """Use a numpy ndarray as the query source for searching on the current flow
+
+        :param array: the numpy ndarray data source
+        :param axis: iterate over that axis
+        :param size: the maximum number of the sub arrays
+        :param shuffle: shuffle the the numpy data source beforehand
+        :param output_fn: the callback function to invoke after indexing
+        :param kwargs: accepts all keyword arguments of `jina client` CLI
+        """
+        from ..clients.python.io import input_numpy
+        self._get_client(**kwargs).search(input_numpy(array, axis, size, shuffle), output_fn)
+
+    def index_lines(self, lines: Iterator[str] = None, filepath: str = None, size: int = None,
+                    sampling_rate: float = None, read_mode='r',
+                    output_fn: Callable[['jina_pb2.Message'], None] = None,
+                    **kwargs):
+        """ Use a list of files as the query source for indexing on the current flow
+
+        :param lines: a list of strings, each is considered as d document
+        :param filepath: a text file that each line contains a document
+        :param size: the maximum number of the documents
+        :param sampling_rate: the sampling rate between [0, 1]
+        :param read_mode: specifies the mode in which the file
+                is opened. 'r' for reading in text mode, 'rb' for reading in binary
+        :param output_fn: the callback function to invoke after indexing
+        :param kwargs: accepts all keyword arguments of `jina client` CLI
+        """
+        from ..clients.python.io import input_lines
+        self._get_client(**kwargs).index(input_lines(lines, filepath,  size, sampling_rate, read_mode), output_fn)
+
+    def index_files(self, patterns: Union[str, List[str]], recursive: bool = True,
+                    size: int = None, sampling_rate: float = None, read_mode: str = None,
+                    output_fn: Callable[['jina_pb2.Message'], None] = None,
+                    **kwargs):
+        """ Use a set of files as the index source for indexing on the current flow
+
+        :param patterns: The pattern may contain simple shell-style wildcards, e.g. '\*.py', '[\*.zip, \*.gz]'
+        :param recursive: If recursive is true, the pattern '**' will match any files and
+                    zero or more directories and subdirectories.
+        :param size: the maximum number of the files
+        :param sampling_rate: the sampling rate between [0, 1]
+        :param read_mode: specifies the mode in which the file
+                is opened. 'r' for reading in text mode, 'rb' for reading in
+        :param output_fn: the callback function to invoke after indexing
+        :param kwargs: accepts all keyword arguments of `jina client` CLI
+        """
+        from ..clients.python.io import input_files
+        self._get_client(**kwargs).index(input_files(patterns, recursive, size, sampling_rate, read_mode), output_fn)
+
+    def search_files(self, patterns: Union[str, List[str]], recursive: bool = True,
+                     size: int = None, sampling_rate: float = None, read_mode: str = None,
+                     output_fn: Callable[['jina_pb2.Message'], None] = None,
+                     **kwargs):
+        """ Use a set of files as the query source for searching on the current flow
+
+        :param patterns: The pattern may contain simple shell-style wildcards, e.g. '\*.py', '[\*.zip, \*.gz]'
+        :param recursive: If recursive is true, the pattern '**' will match any files and
+                    zero or more directories and subdirectories.
+        :param size: the maximum number of the files
+        :param sampling_rate: the sampling rate between [0, 1]
+        :param read_mode: specifies the mode in which the file
+                is opened. 'r' for reading in text mode, 'rb' for reading in
+        :param output_fn: the callback function to invoke after indexing
+        :param kwargs: accepts all keyword arguments of `jina client` CLI
+        """
+        from ..clients.python.io import input_files
+        self._get_client(**kwargs).search(input_files(patterns, recursive, size, sampling_rate, read_mode), output_fn)
+
+    def search_lines(self, filepath: str = None, lines: Iterator[str] = None, size: int = None,
+                     sampling_rate: float = None, read_mode='r',
+                     output_fn: Callable[['jina_pb2.Message'], None] = None,
+                     **kwargs):
+        """ Use a list of files as the query source for searching on the current flow
+
+        :param filepath: a text file that each line contains a document
+        :param lines: a list of strings, each is considered as d document
+        :param size: the maximum number of the documents
+        :param sampling_rate: the sampling rate between [0, 1]
+        :param read_mode: specifies the mode in which the file
+                is opened. 'r' for reading in text mode, 'rb' for reading in binary
+        :param output_fn: the callback function to invoke after indexing
+        :param kwargs: accepts all keyword arguments of `jina client` CLI
+        """
+        from ..clients.python.io import input_lines
+        self._get_client(**kwargs).search(input_lines(filepath, lines, size, sampling_rate, read_mode), output_fn)
+
+    @deprecated_alias(buffer='input_fn', callback='output_fn')
     def index(self, input_fn: Union[Iterator['jina_pb2.Document'], Iterator[bytes], Callable] = None,
               output_fn: Callable[['jina_pb2.Message'], None] = None,
               **kwargs):
@@ -574,10 +681,8 @@ class Flow:
         .. highlight:: python
         .. code-block:: python
 
-            with f.build(runtime='thread') as flow:
-                flow.index(txt_file='aa.txt')
-                flow.index(image_zip_file='aa.zip', batch_size=64)
-                flow.index(video_zip_file='aa.zip')
+            with f:
+                f.index(input_fn)
                 ...
 
 
@@ -599,17 +704,17 @@ class Flow:
 
         It will start a :py:class:`CLIClient` and call :py:func:`index`.
 
-        :param input_fn: An iterator of bytes. If not given, then you have to specify it in `kwargs`.
+        :param input_fn: An iterator of bytes. If not given, then you have to specify it in **kwargs**.
         :param output_fn: the callback function to invoke after indexing
         :param kwargs: accepts all keyword arguments of `jina client` CLI
         """
         self._get_client(**kwargs).index(input_fn, output_fn)
 
-    @deprecated_alias(raw_bytes='input_fn', callback='output_fn')
+    @deprecated_alias(buffer='input_fn', callback='output_fn')
     def search(self, input_fn: Union[Iterator['jina_pb2.Document'], Iterator[bytes], Callable] = None,
                output_fn: Callable[['jina_pb2.Message'], None] = None,
                **kwargs):
-        """Do indexing on the current flow
+        """Do searching on the current flow
 
         It will start a :py:class:`CLIClient` and call :py:func:`search`.
 
@@ -619,10 +724,8 @@ class Flow:
         .. highlight:: python
         .. code-block:: python
 
-            with f.build(runtime='thread') as flow:
-                flow.search(txt_file='aa.txt')
-                flow.search(image_zip_file='aa.zip', batch_size=64)
-                flow.search(video_zip_file='aa.zip')
+            with f:
+                f.search(input_fn)
                 ...
 
 
@@ -642,14 +745,14 @@ class Flow:
             with f.build(runtime='thread') as flow:
                 flow.search(bytes_gen=my_reader())
 
-        :param input_fn: An iterator of bytes. If not given, then you have to specify it in `kwargs`.
+        :param input_fn: An iterator of bytes. If not given, then you have to specify it in **kwargs**.
         :param output_fn: the callback function to invoke after searching
         :param kwargs: accepts all keyword arguments of `jina client` CLI
         """
         self._get_client(**kwargs).search(input_fn, output_fn)
 
     def dry_run(self, **kwargs):
-        """Send a DRYRUN request to this flow, passing through all pods in this flow
+        """Send a DRYRUN request to this flow, passing through all pods in this flow,
         useful for testing connectivity and debugging"""
         if not self._get_client(**kwargs).dry_run():
             raise FlowConnectivityError('a dry run shows this flow is badly connected due to the network settings')
@@ -681,3 +784,23 @@ class Flow:
     @build_required(FlowBuildLevel.GRAPH)
     def host(self):
         return self._pod_nodes['gateway'].host
+
+    def __iter__(self):
+        return self._pod_nodes.values().__iter__()
+
+    def block(self):
+        """Block the process until user hits KeyboardInterrupt """
+        try:
+            self.logger.success(f'flow is started at {self.host}:{self.port_grpc}, '
+                                f'you can now use client to send request!')
+            threading.Event().wait()
+        except KeyboardInterrupt:
+            pass
+
+    def use_grpc_gateway(self):
+        """Change to use gRPC gateway for IO """
+        self._common_kwargs['rest_api'] = False
+
+    def use_rest_gateway(self):
+        """Change to use REST gateway for IO """
+        self._common_kwargs['rest_api'] = True
