@@ -52,6 +52,49 @@ def set_logger_parser(parser=None):
     return parser
 
 
+def set_hub_base_parser(parser=None):
+    if not parser:
+        parser = set_base_parser()
+    parser.add_argument('--username', type=str, help='the registry username')
+    # _gp = parser.add_mutually_exclusive_group()
+    # _gp.add_argument('--password-stdin', type=argparse.FileType('r'),
+    #                  default=(sys.stdin if sys.stdin.isatty() else None),
+    #                  help='take the password from stdin')
+    parser.add_argument('--password', type=str, help='the plaintext password')
+    parser.add_argument('--registry', type=str, default='https://index.docker.io/v1/',
+                        help='the URL to the registry, e.g. https://index.docker.io/v1/')
+
+    return parser
+
+
+def set_hub_build_parser(parser=None):
+    if not parser:
+        parser = set_base_parser()
+
+    set_hub_base_parser(parser)
+
+    parser.add_argument('path', type=str, help='path to the directory containing '
+                                               'Dockerfile, manifest.yml, README.md'
+                                               'zero or more yaml config, '
+                                               'zero or more Python file. '
+                                               'All files in this directory will be shipped into a Docker image')
+    parser.add_argument('--pull', action='store_true', default=False,
+                        help='downloads any updates to the FROM image in Dockerfiles')
+    parser.add_argument('--push', action='store_true', default=False,
+                        help='push the built image to the registry')
+    return parser
+
+
+def set_hub_pushpull_parser(parser=None):
+    if not parser:
+        parser = set_base_parser()
+
+    set_hub_base_parser(parser)
+
+    parser.add_argument('name', type=str, help='the name of the image.')
+    return parser
+
+
 def set_hw_parser(parser=None):
     if not parser:
         parser = set_base_parser()
@@ -64,6 +107,12 @@ def set_hw_parser(parser=None):
                          'all data, indices, shards and outputs will be saved there')
     gp.add_argument('--logserver', action='store_true', default=False,
                     help='start a log server for the dashboard')
+    gp.add_argument('--logserver-config', type=str,
+                    default=resource_filename('jina',
+                                              '/'.join(('resources', 'logserver.default.yml'))),
+                    help='the yaml config of the log server')
+    gp.add_argument('--download-proxy', type=str,
+                    help='specify the proxy when downloading sample data')
     gp = add_arg_group(parser, 'scalability arguments')
     gp.add_argument('--shards', type=int,
                     default=2,
@@ -126,10 +175,11 @@ def set_flow_parser(parser=None):
 
 
 def set_pea_parser(parser=None):
-    from ..enums import SocketType, PeaRoleType
+    from ..enums import SocketType, PeaRoleType, OnErrorSkip
     from ..helper import random_port, get_random_identity
     from .. import __default_host__
     import os
+    show_all = 'JINA_FULL_CLI' in os.environ
     if not parser:
         parser = set_base_parser()
 
@@ -170,9 +220,9 @@ def set_pea_parser(parser=None):
     gp2.add_argument('--port-out', type=int, default=random_port(),
                      help='port for output data, default a random port between [49152, 65535]')
     gp2.add_argument('--host-in', type=str, default=__default_host__,
-                     help='host address for input, by default it is %s' % __default_host__)
+                     help=f'host address for input, by default it is {__default_host__}')
     gp2.add_argument('--host-out', type=str, default=__default_host__,
-                     help='host address for output, by default it is %s' % __default_host__)
+                     help=f'host address for output, by default it is {__default_host__}')
     gp2.add_argument('--socket-in', type=SocketType.from_string, choices=list(SocketType),
                      default=SocketType.PULL_BIND,
                      help='socket type for input port')
@@ -210,7 +260,7 @@ def set_pea_parser(parser=None):
                      help='comparing the jina and proto version of incoming message with local setup, '
                           'mismatch raise an exception')
     gp5.add_argument('--array-in-pb', action='store_true', default=False,
-                     help='sending raw_bytes and numpy ndarray together within or separately from the protobuf message, '
+                     help='sending buffer and numpy ndarray together within or separately from the protobuf message, '
                           'the latter often yields a better network efficiency')
     gp5.add_argument('--compress-hwm', type=int, default=-1,
                      help='the high watermark that triggers the message compression. '
@@ -220,10 +270,14 @@ def set_pea_parser(parser=None):
                      help='the low watermark that enables the sending of a compressed message. '
                           'compression rate (after_size/before_size) lower than this LWM will be considered as successeful '
                           'compression, and will be sent. Otherwise, it will send the original message without compression')
-    gp5.add_argument('--num-part', type=int, default=1,
-                     help='wait until the number of parts of message are all received')
+    gp5.add_argument('--num-part', type=int, default=0,
+                     **(dict(
+                         help='the number of replicated message sent to the next Pod, 0 and 1 means single part' if show_all else argparse.SUPPRESS)))
     gp5.add_argument('--role', type=PeaRoleType.from_string, choices=list(PeaRoleType),
                      help='the role of this pea in a pod')
+    gp5.add_argument('--skip-on-error', type=OnErrorSkip.from_string, choices=list(OnErrorSkip),
+                     default=OnErrorSkip.NONE,
+                     help='skip strategy on error message. ')
 
     gp6 = add_arg_group(parser, 'pea EXPERIMENTAL arguments')
     gp6.add_argument('--memory-hwm', type=int, default=-1,
@@ -268,7 +322,7 @@ def set_pod_parser(parser=None):
     gp4.add_argument('--scheduling', type=SchedulerType.from_string, choices=list(SchedulerType),
                      default=SchedulerType.LOAD_BALANCE,
                      help='the strategy of scheduling workload among peas')
-    gp4.add_argument('--reducing-yaml-path', type=str, default='_forward',
+    gp4.add_argument('--reducing-yaml-path', type=str, default='_merge',
                      help='the executor used for reducing the result from all replicas, '
                           'accepted type follows "--yaml-path"')
     gp4.add_argument('--shutdown-idle', action='store_true', default=False,
@@ -333,7 +387,7 @@ def _set_grpc_parser(parser=None):
     from .. import __default_host__
     gp1 = add_arg_group(parser, 'grpc and remote arguments')
     gp1.add_argument('--host', type=str, default=__default_host__,
-                     help='host address of the pea/gateway, by default it is %s.' % __default_host__)
+                     help=f'host address of the pea/gateway, by default it is {__default_host__}.')
     gp1.add_argument('--port-grpc',
                      type=int,
                      default=random_port(),
@@ -390,6 +444,11 @@ def set_gateway_parser(parser=None):
                      help='the number of additional requests to fetch on every receive')
     gp1.add_argument('--allow-spawn', action='store_true', default=False,
                      help='accept the spawn requests sent from other remote Jina')
+    gp1.add_argument('--rest-api', action='store_true', default=False,
+                     help='use REST-API as the interface instead of gRPC with port number '
+                          'set to the value of "port-grpc"')
+    # gp1.add_argument('--to-datauri', action='store_true', default=False,
+    #                  help='always represent the result document with data URI, instead of using buffer/blob/text')
     return parser
 
 
@@ -397,6 +456,8 @@ def set_client_cli_parser(parser=None):
     if not parser:
         parser = set_base_parser()
 
+    from ..enums import ClientMode
+
     _set_grpc_parser(parser)
 
     gp1 = add_arg_group(parser, 'client-specific arguments')
@@ -404,14 +465,14 @@ def set_client_cli_parser(parser=None):
 
     gp1.add_argument('--batch-size', type=int, default=100,
                      help='the number of documents in each request')
-    gp1.add_argument('--mode', choices=['index', 'search', 'train'], type=str,
+    gp1.add_argument('--mode', choices=list(ClientMode), type=ClientMode.from_string,
                      # required=True,
                      help='the mode of the client and the server')
     gp1.add_argument('--top-k', type=int,
                      default=10,
                      help='top_k results returned in the search mode')
-    gp1.add_argument('--in-proto', action='store_true', default=False,
-                     help='if the input data is already in protobuf Document format, or in raw bytes')
+    gp1.add_argument('--mime-type', type=str,
+                     help='MIME type of the input, useful when input-type is set to BUFFER')
     gp1.add_argument('--callback-on-body', action='store_true', default=False,
                      help='callback function works directly on the request body')
     gp1.add_argument('--first-request-id', type=int,
@@ -422,9 +483,10 @@ def set_client_cli_parser(parser=None):
                      help='the starting number of doc_id, the consequent doc_id will increment by one')
     _gp.add_argument('--random-doc-id', action='store_true', default=False,
                      help='randomize the doc_id, if this is set then `first_request_id` is ignored')
-
     gp1.add_argument('--timeout-ready', type=int, default=10000,
                      help='timeout (ms) of a pea is ready for request, -1 for waiting forever')
+    gp1.add_argument('--filter-by', type=str, nargs='*',
+                     help='field names to search on')
     return parser
 
 
@@ -464,6 +526,29 @@ def get_main_parser():
                       description='Check the import status all executors and drivers',
                       formatter_class=_chf))
 
+    pp = sp.add_parser('hub', help='build, push, pull Jina Hub images',
+                       description='Build, push, pull Jina Hub images',
+                       formatter_class=_chf)
+
+    spp = pp.add_subparsers(dest='hub',
+                            description='use "%(prog)-8s [sub-command] --help" '
+                                        'to get detailed information about each sub-command', required=True)
+
+    set_hub_build_parser(
+        spp.add_parser('build', help='build a directory into Jina hub image',
+                       description='Build a directory into Jina hub image',
+                       formatter_class=_chf))
+
+    set_hub_pushpull_parser(
+        spp.add_parser('push', help='push an image to the Jina hub registry',
+                       description='Push an image to the Jina hub registry',
+                       formatter_class=_chf))
+
+    set_hub_pushpull_parser(
+        spp.add_parser('pull', help='pull an image from the Jina hub registry to local',
+                       description='Pull an image to the Jina hub registry to local',
+                       formatter_class=_chf))
+
     set_pea_parser(sp.add_parser('pea',
                                  description='Start a Jina pea. '
                                              'You should rarely use this directly unless you '
