@@ -6,20 +6,22 @@ import os
 import threading
 
 import grpc
+from google.protobuf.json_format import MessageToJson
 
-from jina.logging.profile import TimeContext
 from .grpc_asyncio import AsyncioExecutor
+from .pea import BasePea
 from .zmq import AsyncZmqlet, add_envelope
 from .. import __stop_msg__
-from ..excepts import NoExplicitMessage, RequestLoopEnd, NoDriverForRequest, BadRequestType
-from ..executors import BaseExecutor
+from ..enums import ClientMode
+from ..excepts import NoExplicitMessage, RequestLoopEnd, NoDriverForRequest, BadRequestType, GatewayPartialMessage
 from ..logging.base import get_logger
+from ..logging.profile import TimeContext
 from ..main.parser import set_pea_parser, set_pod_parser
 from ..proto import jina_pb2_grpc, jina_pb2
 
 
 class GatewayPea:
-    """A :class:`BasePea`-like class for holding Gateway.
+    """A :class:`BasePea`-like class for holding a gRPC Gateway.
 
     It has similar :meth:`start` and context interface as :class:`BasePea`,
     but it is not built on thread or process. It works directly in the main thread main process.
@@ -34,27 +36,31 @@ class GatewayPea:
             os.unsetenv('http_proxy')
             os.unsetenv('https_proxy')
 
+        os.environ['JINA_POD_NAME'] = 'gateway'
         self.logger = get_logger(self.__class__.__name__, **vars(args))
         if args.allow_spawn:
             self.logger.critical('SECURITY ALERT! this gateway allows SpawnRequest from remote Jina')
+        self._p_servicer = self._Pea(args)
+        self._stop_event = threading.Event()
+        self.is_ready = threading.Event()
+        self.init_server(args)
+
+    def init_server(self, args):
         self._server = grpc.server(
             AsyncioExecutor(),
             options=[('grpc.max_send_message_length', args.max_message_size),
                      ('grpc.max_receive_message_length', args.max_message_size)])
 
-        self._p_servicer = self._Pea(args)
         jina_pb2_grpc.add_JinaRPCServicer_to_server(self._p_servicer, self._server)
-        self._bind_address = '{0}:{1}'.format(args.host, args.port_grpc)
+        self._bind_address = f'{args.host}:{args.port_grpc}'
         self._server.add_insecure_port(self._bind_address)
-        self._stop_event = threading.Event()
-        self.is_ready = threading.Event()
 
     def __enter__(self):
         return self.start()
 
     def start(self):
         self._server.start()
-        self.logger.success('gateway is listening at: %s' % self._bind_address)
+        self.logger.success(f'gateway is listening at: {self._bind_address}')
         self._stop_event.clear()
         self.is_ready.set()
         return self
@@ -81,15 +87,41 @@ class GatewayPea:
             self.args = args
             self.name = args.name or self.__class__.__name__
             self.logger = get_logger(self.name, **vars(args))
-            self.executor = BaseExecutor()
-            self.executor.attach(pea=self)
+            # self.executor = BaseExecutor()
+            # if args.to_datauri:
+            #     from ..drivers.convert import All2URI
+            #     for k in ['SearchRequest', 'IndexRequest', 'TrainRequest']:
+            #         self.executor.add_driver(All2URI(), k)
+            # self.executor.attach(pea=self)
             self.peapods = []
 
-        def recv_callback(self, msg):
+        @property
+        def message(self) -> 'jina_pb2.Message':
+            """Get the current protobuf message to be processed"""
+            return self._message
+
+        @property
+        def request_type(self) -> str:
+            return self._request.__class__.__name__
+
+        @property
+        def request(self) -> 'jina_pb2.Request':
+            """Get the current request body inside the protobuf message"""
+            return self._request
+
+        def handle(self, msg: 'jina_pb2.Message'):
             try:
-                return self.executor(msg.__class__.__name__)
+                msg.request.status.CopyFrom(msg.envelope.status)
+                self._request = getattr(msg.request, msg.request.WhichOneof('body'))
+                self._message = msg
+                if msg.envelope.num_part != [1]:
+                    raise GatewayPartialMessage(f'gateway can not handle message with num_part={msg.envelope.num_part}')
+                # self.executor(self.request_type)
+                # envelope will be dropped when returning to the client
+                return msg.request
             except NoExplicitMessage:
-                self.logger.error('gateway should not receive partial message, it can not do reduce')
+                self.logger.error('gateway should not receive partial message, it can not do reduce. '
+                                  'maybe you forget to add .join() as the last step of the flow?')
             except RequestLoopEnd:
                 self.logger.error('event loop end signal should not be raised in the gateway')
             except NoDriverForRequest:
@@ -98,8 +130,9 @@ class GatewayPea:
 
         async def CallUnary(self, request, context):
             with AsyncZmqlet(self.args, logger=self.logger) as zmqlet:
-                await zmqlet.send_message(add_envelope(request, 'gateway', zmqlet.args.identity))
-                return await zmqlet.recv_message(callback=self.recv_callback)
+                await zmqlet.send_message(add_envelope(request, 'gateway', zmqlet.args.identity,
+                                                       num_part=self.args.num_part))
+                return await zmqlet.recv_message(callback=self.handle)
 
         async def Call(self, request_iterator, context):
             with AsyncZmqlet(self.args, logger=self.logger) as zmqlet:
@@ -113,8 +146,9 @@ class GatewayPea:
                         try:
                             asyncio.create_task(
                                 zmqlet.send_message(
-                                    add_envelope(next(request_iterator), 'gateway', zmqlet.args.identity)))
-                            fetch_to.append(asyncio.create_task(zmqlet.recv_message(callback=self.recv_callback)))
+                                    add_envelope(next(request_iterator), 'gateway', zmqlet.args.identity,
+                                                 num_part=self.args.num_part)))
+                            fetch_to.append(asyncio.create_task(zmqlet.recv_message(callback=self.handle)))
                         except StopIteration:
                             return True
                     return False
@@ -123,6 +157,11 @@ class GatewayPea:
                     self.logger.warning('if this takes too long, you may want to take smaller "--prefetch" or '
                                         'ask client to reduce "--batch-size"')
                     is_req_empty = prefetch_req(self.args.prefetch, prefetch_task)
+                    if is_req_empty and not prefetch_task:
+                        self.logger.error('receive an empty stream from the client! '
+                                          'please check your client\'s input_fn, '
+                                          'you can use "PyClient.check_input(input_fn())"')
+                        return
 
                 while not (zmqlet.msg_sent == zmqlet.msg_recv != 0 and is_req_empty):
                     self.logger.info(f'send: {zmqlet.msg_sent} '
@@ -169,7 +208,8 @@ class GatewayPea:
                 warn_msg = f'the gateway at {self.args.host}:{self.args.port_grpc} ' \
                            f'does not support remote spawn, please restart it with --allow-spawn'
                 request.log_record = warn_msg
-                request.status = jina_pb2.SpawnRequest.ERROR_NOTALLOWED
+                request.status.code = jina_pb2.Status.ERROR_NOTALLOWED
+                request.status.description = warn_msg
                 self.logger.warning(warn_msg)
                 for j in range(1):
                     yield request
@@ -177,3 +217,83 @@ class GatewayPea:
         def close(self):
             for p in self.peapods:
                 p.close()
+
+
+class RESTGatewayPea(BasePea):
+    """A :class:`BasePea`-like class for holding a HTTP Gateway.
+
+    :class`RESTGatewayPea` is still in beta. Feature such as prefetch is not available yet.
+    Unlike :class:`GatewayPea`, it does not support bi-directional streaming. Therefore, it is
+    synchronous from the client perspective.
+    """
+
+    def loop_body(self):
+        self._p_servicer = GatewayPea._Pea(self.args)
+        self.get_http_server()
+
+    def close(self):
+        if hasattr(self, 'terminate'):
+            self.terminate()
+
+    def get_http_server(self):
+        try:
+            from flask import Flask, Response, jsonify, request
+            from flask_cors import CORS, cross_origin
+            from gevent.pywsgi import WSGIServer
+        except ImportError:
+            raise ImportError('Flask or its dependencies are not fully installed, '
+                              'they are required for serving HTTP requests.'
+                              'Please use pip install "jina[http]" to install it.')
+        app = Flask(__name__)
+        app.config['CORS_HEADERS'] = 'Content-Type'
+        CORS(app)
+
+        def http_error(reason, code):
+            return jsonify({'reason': reason}), code
+
+        @app.route('/ready')
+        @cross_origin()
+        def is_ready():
+            return Response(status=200)
+
+        @app.route('/api/<mode>', methods=['POST'])
+        @cross_origin()
+        def api(mode):
+            from ..clients import python
+            mode_fn = getattr(python.request, mode, None)
+            if mode_fn is None:
+                return http_error(f'mode: {mode} is not supported yet', 405)
+            content = request.json
+            if 'data' not in content:
+                return http_error('"data" field is empty', 406)
+
+            content['mode'] = ClientMode.from_string(mode)
+
+            results = get_result_in_json(getattr(python.request, mode)(**content))
+            return Response(asyncio.run(results),
+                            status=200,
+                            mimetype='application/json')
+
+        async def get_result_in_json(req_iter):
+            return [MessageToJson(k) async for k in self._p_servicer.Call(req_iter, None)]
+
+        # os.environ['WERKZEUG_RUN_MAIN'] = 'true'
+        # log = logging.getLogger('werkzeug')
+        # log.disabled = True
+        # app.logger.disabled = True
+
+        # app.run('0.0.0.0', 5000)
+        server = WSGIServer((self.args.host, self.args.port_grpc), app, log=None)
+
+        def close(*args, **kwargs):
+            server.stop()
+            self.unset_ready()
+            self.is_shutdown.set()
+
+        from gevent import signal
+        signal.signal(signal.SIGTERM, close)
+        signal.signal(signal.SIGINT, close)  # CTRL C
+        self.set_ready()
+        self.logger.warning('you are using a REST gateway, which is still in early beta version. '
+                            'advanced features such as prefetch and streaming are disabled.')
+        server.serve_forever()
