@@ -5,6 +5,7 @@ import asyncio
 import os
 import sys
 import tempfile
+import time
 from typing import List, Callable, Optional
 from typing import Tuple
 
@@ -45,7 +46,7 @@ class Zmqlet:
                 self.logger.success(f'compression is enabled and the high watermark is {args.compress_hwm} bytes')
             except ModuleNotFoundError:
                 self.logger.error(f'compression is enabled but you do not have lz4 package. '
-                                  f'use "pip install lz4" to install this dependency')
+                                  f'use pip install "jina[lz4]" to install this dependency')
                 args.compress_hwm = -1  # disable the compression
         self.send_recv_kwargs = vars(args)
         self.ctrl_addr, self.ctrl_with_ipc = self.get_ctrl_address(args)
@@ -123,17 +124,17 @@ class Zmqlet:
                                                     use_ipc=self.ctrl_with_ipc)
             else:
                 ctrl_sock, ctrl_addr = _init_socket(ctx, __default_host__, self.args.port_ctrl, SocketType.PAIR_BIND)
-            self.logger.debug('control over %s' % (colored(ctrl_addr, 'yellow')))
+            self.logger.debug(f'control over {colored(ctrl_addr, "yellow")}')
             self.opened_socks.append(ctrl_sock)
 
             in_sock, in_addr = _init_socket(ctx, self.args.host_in, self.args.port_in, self.args.socket_in,
                                             self.args.identity)
-            self.logger.debug('input %s:%s' % (self.args.host_in, colored(self.args.port_in, 'yellow')))
+            self.logger.debug(f'input {self.args.host_in}:{colored(self.args.port_in, "yellow")}')
             self.opened_socks.append(in_sock)
 
             out_sock, out_addr = _init_socket(ctx, self.args.host_out, self.args.port_out, self.args.socket_out,
                                               self.args.identity)
-            self.logger.debug('output %s:%s' % (self.args.host_out, colored(self.args.port_out, 'yellow')))
+            self.logger.debug(f'output {self.args.host_out}:{colored(self.args.port_out, "yellow")}')
             self.opened_socks.append(out_sock)
 
             self.logger.info(
@@ -150,7 +151,7 @@ class Zmqlet:
         return zmq.Context()
 
     def __enter__(self):
-        # time.sleep(.1)  # sleep a bit until everything is ready
+        # time.sleep(.1)  # timeout handshake is unnecessary at the Pod level, it is only required for gateway
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
@@ -262,6 +263,10 @@ class AsyncZmqlet(Zmqlet):
         except (asyncio.CancelledError, TypeError) as ex:
             self.logger.error(f'{ex}, gateway cancelled?')
 
+    def __enter__(self):
+        time.sleep(.2)  # sleep a bit until handshake is done
+        return self
+
 
 def send_ctrl_message(address: str, cmd: 'jina_pb2.Request.ControlRequest', timeout: int):
     """Send a control message to a specific address and wait for the response
@@ -334,7 +339,7 @@ def _prep_send_msg(array_in_pb, compress_hwm, compress_lwm, msg, sock, timeout):
         _msg, num_bytes = _prepare_send_msg(c_id, [msg.SerializeToString()], compress_hwm, compress_lwm)
     else:
         doc_bytes, chunk_bytes, chunk_byte_type = _extract_bytes_from_msg(msg)
-        # now raw_bytes are removed from message, hoping for faster de/serialization
+        # now buffer are removed from message, hoping for faster de/serialization
         _msg = [msg.SerializeToString(),  # 1
                 chunk_byte_type,  # 2
                 b'%d' % len(doc_bytes), b'%d' % len(chunk_bytes),  # 3, 4
@@ -508,7 +513,7 @@ def _prepare_recv_msg(sock, msg_data, check_version: bool):
 
     # now we have a barebone msg, we need to fill in data
     if len(msg_data) > 3:
-        _fill_raw_bytes_to_msg(msg, msg_data, offset=3)
+        _fill_buffer_to_msg(msg, msg_data, offset=3)
 
     return msg, num_bytes
 
@@ -518,11 +523,11 @@ def _get_random_ipc() -> str:
     try:
         tmp = os.environ['JINA_IPC_SOCK_TMP']
         if not os.path.exists(tmp):
-            raise ValueError('This directory for sockets ({}) does not seems to exist.'.format(tmp))
+            raise ValueError(f'This directory for sockets ({tmp}) does not seems to exist.')
         tmp = os.path.join(tmp, get_random_identity())
     except KeyError:
         tmp = tempfile.NamedTemporaryFile().name
-    return 'ipc://%s' % tmp
+    return f'ipc://{tmp}'
 
 
 def _init_socket(ctx: 'zmq.Context', host: str, port: int,
@@ -556,10 +561,10 @@ def _init_socket(ctx: 'zmq.Context', host: str, port: int,
             # JEP2, if it is bind, then always bind to local
             if host != __default_host__:
                 default_logger.warning(
-                    'host is set from %s to %s as the socket is in BIND type' % (host, __default_host__))
+                    f'host is set from {host} to {__default_host__} as the socket is in BIND type')
                 host = __default_host__
             if port is None:
-                sock.bind_to_random_port('tcp://%s' % host)
+                sock.bind_to_random_port(f'tcp://{host}')
             else:
                 try:
                     sock.bind('tcp://%s:%d' % (host, port))
@@ -627,8 +632,8 @@ def _extract_bytes_from_msg(msg: 'jina_pb2.Message') -> Tuple:
     docs = msg.request.train.docs or msg.request.index.docs or msg.request.search.docs
     # for train request
     for d in docs:
-        doc_bytes.append(d.raw_bytes)
-        d.ClearField('raw_bytes')
+        doc_bytes.append(d.buffer)
+        d.ClearField('buffer')
 
         for c in d.chunks:
             # oneof content {
@@ -636,17 +641,17 @@ def _extract_bytes_from_msg(msg: 'jina_pb2.Message') -> Tuple:
             # NdArray blob = 3;
             # bytes raw = 7;
             # }
-            chunk_bytes.append(c.embedding.raw_bytes)
-            c.embedding.ClearField('raw_bytes')
+            chunk_bytes.append(c.embedding.buffer)
+            c.embedding.ClearField('buffer')
 
             ctype = c.WhichOneof('content') or ''
             chunk_byte_type = ctype.encode()
-            if ctype == 'raw_bytes':
-                chunk_bytes.append(c.raw_bytes)
-                c.ClearField('raw_bytes')
+            if ctype == 'buffer':
+                chunk_bytes.append(c.buffer)
+                c.ClearField('buffer')
             elif ctype == 'blob':
-                chunk_bytes.append(c.blob.raw_bytes)
-                c.blob.ClearField('raw_bytes')
+                chunk_bytes.append(c.blob.buffer)
+                c.blob.ClearField('buffer')
             elif ctype == 'text':
                 chunk_bytes.append(c.text.encode())
                 c.ClearField('text')
@@ -654,7 +659,7 @@ def _extract_bytes_from_msg(msg: 'jina_pb2.Message') -> Tuple:
     return doc_bytes, chunk_bytes, chunk_byte_type
 
 
-def _fill_raw_bytes_to_msg(msg: 'jina_pb2.Message', msg_data: List[bytes], offset: int = 2):
+def _fill_buffer_to_msg(msg: 'jina_pb2.Message', msg_data: List[bytes], offset: int = 2):
     chunk_byte_type = msg_data[offset].decode()
     doc_bytes_len = int(msg_data[offset + 1])
     chunk_bytes_len = int(msg_data[offset + 2])
@@ -670,19 +675,19 @@ def _fill_raw_bytes_to_msg(msg: 'jina_pb2.Message', msg_data: List[bytes], offse
     docs = msg.request.train.docs or msg.request.index.docs or msg.request.search.docs
     for d in docs:
         if doc_bytes and doc_bytes[d_idx]:
-            d.raw_bytes = doc_bytes[d_idx]
+            d.buffer = doc_bytes[d_idx]
             d_idx += 1
 
         for c in d.chunks:
             if chunk_bytes and chunk_bytes[c_idx]:
-                c.embedding.raw_bytes = chunk_bytes[c_idx]
+                c.embedding.buffer = chunk_bytes[c_idx]
             c_idx += 1
 
-            if chunk_byte_type == 'raw_bytes':
-                c.raw_bytes = chunk_bytes[c_idx]
+            if chunk_byte_type == 'buffer':
+                c.buffer = chunk_bytes[c_idx]
                 c_idx += 1
             elif chunk_byte_type == 'blob':
-                c.blob.raw_bytes = chunk_bytes[c_idx]
+                c.blob.buffer = chunk_bytes[c_idx]
                 c_idx += 1
             elif chunk_byte_type == 'text':
                 c.text = chunk_bytes[c_idx].decode()
@@ -710,12 +715,13 @@ def _add_route(evlp, pod_name, identity):
     r.pod_id = identity
 
 
-def add_envelope(req, pod_name, identity) -> 'jina_pb2.Message':
+def add_envelope(req, pod_name, identity, num_part=1) -> 'jina_pb2.Message':
     """Add envelope to a request and make it as a complete message, which can be transmitted between pods.
 
     :param req: the protobuf request
     :param pod_name: the name of the current pod
     :param identity: the identity of the current pod
+    :param num_part: the total parts of the message, 0 and 1 means single part
     :return: the resulted protobuf message
     """
     msg = jina_pb2.Message()
@@ -728,6 +734,9 @@ def add_envelope(req, pod_name, identity) -> 'jina_pb2.Message':
     _add_version(msg.envelope)
     _add_route(msg.envelope, pod_name, identity)
     msg.request.CopyFrom(req)
+    msg.envelope.num_part.append(1)
+    if num_part > 1:
+        msg.envelope.num_part.append(num_part)
     return msg
 
 
